unit ZReport;

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
     ExtCtrls, StdCtrls, DB, DBTables, Printers, DsgnIntf, ZRPrntr, ZREval,
     ZREscape;

type
  TZRBandType = (rbTitle,
                 rbPageHeader,
                 rbDetail,
                 rbPageFooter,
                 rbSummary,
                 rbGroupHeader,
                 rbGroupFooter,
                 rbSubDetail,
                 rbColumnHeader,
                 rbColumnFooter,
                 rbChild);
  TZRBandTypes = set of TZRBandType;

const
  {    design-time }
  clReportFrame    : TColor = clWindowText;
  clReportControl  : TColor = clGray+$303030;
  clReportBand     : TColor = TColor($C0DCC0+$101010);
  clReport         : TColor = clWindow;
  clReportGrid     : TColor = clBtnShadow;
  DefaultFontName  : String = 'Courier New';

type
  { Forward declarations }
  TZReportControl   = class;
  TZReport          = class;
  TZRCustomBand     = class;
//  TZRChildBand      = class;
  TZRSubDetailBand  = class;

  { TZRFrame }
  TZRFrame = class(TPersistent)
    private
      fOwner : TZReportControl;
      fValues: array[0..3] of Integer;
      function  GetValue(Index: Integer): Integer;
      procedure SetValue(Index: Integer; Value: Integer);
    public
      constructor Create(AOwner: TZReportControl);
      procedure Paint;
      procedure Print(X, Y: Integer);
    published
      property Left   : Integer index 0 read GetValue write SetValue default 0;
      property Top    : Integer index 1 read GetValue write SetValue default 0;
      property Right  : Integer index 2 read GetValue write SetValue default 0;
      property Bottom : Integer index 3 read GetValue write SetValue default 0;
      property Width  : Integer index 4 read GetValue;
      property Height : Integer index 5 read GetValue;
  end;

  { TZReportControl }
  TZRBeforePrintEvent = procedure (Sender : TObject; var DoPrint : Boolean) of object;
  TZRAfterPrintEvent  = procedure (Sender : TObject; Printed : Boolean) of object;
  TZROnNeedDataEvent  = procedure (Sender : TObject; var MoreData : Boolean) of object;

  TZReportControl = class(TCustomControl)
    private
      { Private declarations }
      fAlignment  : TAlignment;
      fAlignToBand: Boolean;
      fReport     : TZReport;
      fIsPrinting : Boolean;
      fLength     : Integer;
      fBeforePrint: TZRBeforePrintEvent;
      fAfterPrint : TZRAfterPrintEvent;
      fDoPrint    : Boolean;
      fFontStyle  : TEscapeStyles;
      fTextResize : Boolean;
      fFrame      : TZRFrame;
      procedure SetAlignment(Value: TAlignment);
      procedure SetAlignToBand(Value : Boolean);
      function  GetLeft: Integer;
      procedure SetLeft(Value: Integer);
      function  GetTop: Integer;
      procedure SetTop(Value: Integer);
      function  GetWidth: Integer;
      procedure SetWidth(Value: Integer);
      function  GetHeight: Integer;
      procedure SetHeight(Value: Integer);
      function  GetCharSize: TPoint; 
      procedure SetFontStyle(Value: TEscapeStyles);
      procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
      function GetFramed(Index: Integer): Integer;
    protected
      { Protected declarations }
      procedure Loaded; override;
      procedure Paint; override;
      procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
      procedure SetParent(AParent: TWinControl); override;
      procedure AdjustBounds; virtual;
      procedure SetCharBounds(ALeft, ATop, AWidth, AHeight: Integer); virtual;
      procedure DrawString(X, Y: Integer; S: String);
      procedure Prepare; virtual;
      procedure Unprepare; virtual;
      procedure DoBeforePrint(var DoPrint: Boolean); virtual;
      procedure DoAfterPrint(Printed: Boolean) ; virtual;
      function  PrintString(X, Y: Integer; S: String; Styles: TEscapeStyles): Boolean;
      procedure PrintTo(ParentX, ParentY: Integer); virtual;
      procedure MasterDataAdvance; virtual;
      property Alignment  : TAlignment read fAlignment write SetAlignment default taLeftJustify;
      property AlignToBand: Boolean read fAlignToBand write SetAlignToBand;
      property CharSize   : TPoint read GetCharSize;
      property FontStyle  : TEscapeStyles read fFontStyle write SetFontStyle;
      property IsPrinting : Boolean read fIsPrinting write fIsPrinting;
      property Length     : Integer read fLength write fLength;
      property BeforePrint: TZRBeforePrintEvent read fBeforePrint write fBeforePrint;
      property AfterPrint : TZRAfterPrintEvent  read fAfterPrint  write fAfterPrint;
    public
      { Public declarations }
      constructor Create(AOwner: TComponent); override;
      destructor  Destroy; override;
      property Canvas;
      property Enabled;
      property Report: TZReport read fReport;
      property Frame : TZRFrame read fFrame write fFrame;
      property FramedLeft  : Integer index 0 read GetFramed;
      property FramedTop   : Integer index 1 read GetFramed;
      property FramedWidth : Integer index 2 read GetFramed;
      property FramedHeight: Integer index 3 read GetFramed;
    published
      { Published declarations }
      property Left  : Integer read GetLeft   write SetLeft;
      property Top   : Integer read GetTop    write SetTop;
      property Width : Integer read GetWidth  write SetWidth;
      property Height: Integer read GetHeight write SetHeight;
  end;
  TZReportControlClass = class of TZReportControl;

  { TZRCustomBand }
  TZRCustomBand = class(TZReportControl)
    private
      fBandType      : TZRBandType;
//      fAlignToBottom : Boolean;
      fParentResizing: Boolean;
      fForceNewColumn: Boolean;
      fForceNewPage  : Boolean;
      fParentBand    : TZRCustomBand;
      procedure SetBandType(Value : TZRBandType);
      function  GetChild    : TZRCustomBand;
      function  GetHasChild : Boolean;
      procedure SetHasChild(Value : Boolean);
//      procedure SetParentBand(Value : TZRCustomBand);
      procedure PaintGrid;
    protected
      procedure CreateParams(var Params: TCreateParams); override;
      procedure Notification(AComponent: TComponent; Operation: TOperation); override;
      procedure SetParent(AParent: TWinControl); override;
      procedure Paint; override;
      procedure SetCharBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
      procedure Prepare; override;
      procedure Unprepare; override;
      procedure DoBeforePrint(var DoPrint: Boolean); override;
      procedure DoAfterPrint(Printed: Boolean); override;
      procedure Print; virtual;
      procedure RegisterBands; virtual;
    public
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
//      property AlignToBottom : Boolean read fAlignToBottom write fAlignToBottom;
      property BandType  : TZRBandType read fBandType write SetBandType;
      property ChildBand : TZRCustomBand read GetChild;
      property HasChild  : Boolean read GetHasChild write SetHasChild stored False;
      property ParentBand : TZRCustomBand read fParentBand;
      property ForceNewColumn : Boolean read fForceNewColumn write fForceNewColumn;
      property ForceNewPage   : Boolean read fForceNewPage   write fForceNewPage;
      property Length;
    published
      property AfterPrint;
      property BeforePrint;
      property Enabled;
  end;

  { TZRBand }
  TZRBand = class(TZRCustomBand)
    published
//      property AlignToBottom;
      property BandType;
      property Frame;
      property HasChild;
      property ForceNewColumn;
      property ForceNewPage;
  end;

  { TZRController }
  TZRControllerBand = class(TZRCustomBand)
    private
      fMaster        : TZRControllerBand;
      fDataSet       : TDataSet;
      fDetail        : TZRCustomBand;
      fFooter        : TZRCustomBand;
      fHeader        : TZRCustomBand;
      fDetailNumber  : Integer;
      fPrintBefore   : Boolean;
      fPrintIfEmpty  : Boolean;
      GroupList      : TList;
      PrintAfterList : TList;
      PrintBeforeList: TList;
      NotifyList     : TList;
      fOnNeedData    : TZROnNeedDataEvent;
      procedure SetMaster(Value : TZRControllerBand);
      procedure SetDataSet(Value : TDataSet);
      procedure SetPrintBefore(Value : Boolean); virtual;
    protected
      procedure Notification(AComponent: TComponent; Operation: TOperation); override;
      procedure SetParent(AParent: TWinControl); override;
      procedure RegisterBands; override;
      procedure Print; override;
      procedure Prepare; override;
      procedure Unprepare; override;
      procedure MasterDataAdvance; override;
      procedure BuildTree;
    public
      constructor Create(AOwner : TComponent); override;
      destructor  Destroy; override;
      procedure AddNotifyClient(Value : TZReportControl);
      property DataSet     : TDataSet read fDataSet write SetDataSet;
      property DetailNumber: Integer read fDetailNumber;
      property Detail      : TZRCustomBand read fDetail write fDetail;
      property Footer      : TZRCustomband read fFooter write fFooter;
      property Header      : TZRCustomBand read fHeader write fHeader;
      property Master      : TZRControllerBand read fMaster write SetMaster;
      property PrintBefore : Boolean read fPrintBefore write SetPrintBefore;
      property PrintIfEmpty: Boolean read fPrintIfEmpty write fPrintIfEmpty;
    published
      property OnNeedData: TZROnNeedDataEvent read fOnNeedData write fOnNeedData;
  end;

  { TZRGroup }
  TZRGroup = class(TZRCustomBand)
    private
      fMaster     : TZRControllerBand;
      fFooterBand : TZRCustomBand;
      fExpression : String;
      fEvaluator  : TZREvaluator;
      fReprint    : Boolean;
      GroupValue  : TZREvValue;
      HasResult   : Boolean;
      procedure SetExpression(Value : String);
      function  GetHasFooter: Boolean;
      procedure SetHasFooter(Value: Boolean);
      procedure SetFooterBand(Value : TZRCustomBand);
      procedure SetMaster(Value: TZRControllerBand);
    protected
      procedure Check;
      procedure Notification(AComponent: TComponent; Operation: TOperation); override;
      procedure SetParent(AParent: TWinControl); override;
      procedure Prepare; override;
      procedure Unprepare; override;
      property Reprint: Boolean read fReprint write fReprint;
    public
      constructor Create(AOwner : TComponent); override;
      destructor  Destroy; override;
    published
//      property AlignToBottom;
      property HasChild;
      property HasFooter : Boolean read GetHasFooter write SetHasFooter stored False;
      property Expression: String  read fExpression write SetExpression;
      property FooterBand: TZRCustomBand read fFooterBand write SetFooterBand;
      property Master    : TZRControllerBand read fMaster write SetMaster;
      property ForceNewColumn;
      property ForceNewPage;
      property Frame;
  end;

  { TZRSubDetailGroupBands }
  TZRSubDetailBands = class(TPersistent)
    private
      Owner : TZRSubDetailBand;
    protected
      function  GetFooterBand : TZRCustomBand;
      function  GetHeaderBand : TZRCustomBand;
      function  GetHasFooter : Boolean;
      function  GetHasHeader : Boolean;
      procedure SetHasFooter(Value : Boolean);
      procedure SetHasHeader(Value : Boolean);
    public
      constructor Create(AOwner : TZRSubDetailBand);
      property FooterBand : TZRCustomBand read GetFooterBand;
      property HeaderBand : TZRCustomBand read GetHeaderBand;
    published
      property HasFooter : Boolean read GetHasFooter write SetHasFooter stored False;
      property HasHeader : Boolean read GetHasHeader write SetHasHeader stored False;
  end;

  { TZRSubDetail }

  TZRSubDetailBand = class(TZRControllerBand)
    private
      fBands : TZRSubDetailBands;
      function  GetFooterBand : TZRCustomBand;
      function  GetHeaderBand : TZRCustomBand;
      procedure SetFooterBand(Value : TZRCustomBand);
      procedure SetHeaderBand(Value : TZRCustomBand);
    public
      constructor Create(AOwner : TComponent); override;
      destructor  Destroy; override;
    published
//      property AlignToBottom;
      property Bands: TZRSubDetailBands read fBands write fBands;
      property DataSet;
      property FooterBand: TZRCustomBand read GetFooterBand write SetFooterBand;
      property HeaderBand: TZRCustomBand read GetHeaderBand write SetHeaderBand;
      property HasChild;
      property Master;
      property ForceNewColumn;
      property ForceNewPage;
      property Frame;
      property OnNeedData;
      property PrintBefore;
      property PrintIfEmpty;
  end;

  { TZReportBands }
  TZReportBands = class(TPersistent)
    private
      fOwner : TZReport;
      function  BandInList(BandType : TZRBandType) : TZRCustomBand;
      procedure SetBand(BandType : TZRBandType; Value : Boolean);
      function  GetBand(Index : Integer) : TZRCustomBand;
      function  GetHasBand(Index : Integer) : Boolean;
      procedure SetHasBand(Index : Integer; Value : Boolean);
    public
      constructor Create(AOwner : TZReport);
      property TitleBand        : TZRCustomBand index 1 read GetBand;
      property PageHeaderBand   : TZRCustomBand index 2 read GetBand;
      property ColumnHeaderBand : TZRCustomBand index 3 read GetBand;
      property DetailBand       : TZRCustomBand index 4 read GetBand;
      property ColumnFooterBand : TZRCustomBand index 5 read GetBand;
      property PageFooterBand   : TZRCustomBand index 6 read GetBand;
      property SummaryBand      : TZRCustomBand index 7 read GetBand;
    published
      property HasTitle        : Boolean index 1 read GetHasBand write SetHasBand stored False;
      property HasPageHeader   : Boolean index 2 read GetHasBand write SetHasBand stored False;
      property HasColumnHeader : Boolean index 3 read GetHasBand write SetHasBand stored False;
      property HasDetail       : Boolean index 4 read GetHasBand write SetHasBand stored False;
      property HasColumnFooter : Boolean index 5 read GetHasBand write SetHasBand stored False;
      property HasPageFooter   : Boolean index 6 read GetHasBand write SetHasBand stored False;
      property HasSummary      : Boolean index 7 read GetHasBand write SetHasBand stored False;
  end;

  { TZRMargins }
  TZRMargins = class(TPersistent)
    private
      fOwner  : TZReport;
      fMargins: TRect;
      function  GetValue(Index: Integer): Integer;
      procedure SetValue(Index: Integer; Value: Integer);
    protected
      procedure Changed;
    public
      constructor Create(AOwner: TZReport);
    published
      property Left  : Integer index 0 read GetValue write SetValue;
      property Top   : Integer index 1 read GetValue write SetValue;
      property Right : Integer index 2 read GetValue write SetValue;
      property Bottom: Integer index 3 read GetValue write SetValue;
  end;

  { TZReport }
  TZReport = class(TZRControllerBand)
    private
      { Private declarations }
//      BGThread       : TZRCreateReportThread;
      fAllDataSets   : TList;
      fReady         : Boolean;
      fBands         : TZReportBands;
      fBandList      : TList;
      BandRegList    : TList;
      fMargins       : TZRMargins;
      fColumns       : Integer;
      fColumnSpace   : Integer;
      fColumnTop     : Integer;
      fCurrentX      : Integer;
      fCurrentY      : Integer;
      fCurrentColumn : Integer;
      fLastPage      : Boolean;
      NewColumnForced: Boolean;
      NewPageForced  : Boolean;
      fOptions       : TZRPrinterOptions;
      fPageCount     : Integer;
      fShowProgress  : Boolean;
      fTitle         : String;
      fZRPrinter     : TZRPrinter;
      fAfterPreview  : TNotifyEvent;
      fOnEndPage     : TNotifyEvent;
      fOnStartPage   : TNotifyEvent;
      fOnFormatString: TZRFormatStringEvent;
      procedure SetColumns(Value: Integer);
      procedure SetColumnSpace(Value: Integer);
      function  GetRecordCount: Integer;
      procedure AddBand   (aBand: TZRCustomBand);
      procedure RemoveBand(aBand: TZRCustomBand);
      procedure PrintBand(ABand : TZRCustomBand);
    protected
      { Protected declarations }
      procedure Loaded; override;
      procedure Paint; override;
      procedure RegisterBand(aBand: TZRCustomBand);
      procedure RegisterBands; override;
      function  RemainSpace   : integer;
      function  ColumnWidth   : Integer;
      function  PageFooterSize(Footers: TZRBandTypes): Integer;
      procedure CreateReport(CompositeReport : boolean);
      procedure PreviewFinished(Sender : TObject);
      procedure PrintFinished(Sender : TObject);
    public
      { Public declarations }
      constructor Create(AOwner: TComponent); override;
      destructor  Destroy; override;
      function  CreateBand(BandType : TZRBandType) : TZRBand;
      procedure Print; override;
      procedure Preview;
      procedure RebuildBandList;
      procedure SetCharBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
      procedure SetBandSizes;
      procedure ForceNewColumn;
      procedure ForceNewPage;
      procedure EndPage;
      procedure NewPage;
      procedure EndColumn;
      procedure NewColumn;
      property AllDataSets  : TList   read fAllDataSets write fAllDataSets;
      property Ready        : Boolean read fReady;
      property BandList     : TList   read fBandList;
      property ColumnTop    : Integer read fColumnTop write fColumnTop;
      property CurrentColumn: Integer read fCurrentColumn;
      property CurrentX     : Integer read fCurrentX write fCurrentX;
      property CurrentY     : Integer read fCurrentY write fCurrentY;
      property PageNumber   : Integer read fPageCount;
      property RecordCount  : Integer read GetRecordCount;
      property ZRPrinter    : TZRPrinter read fZRPrinter write fZRPrinter;
    published
      { Published declarations }
      property Bands         : TZReportBands read fBands write fBands;
      property Columns       : Integer read fColumns     write SetColumns;
      property ColumnSpace   : Integer read fColumnSpace write SetColumnSpace;
      property DataSet;
      property Font;
      property Margins       : TZRMargins read fMargins write fMargins;
      property Options       : TZRPrinterOptions read fOptions write fOptions;
      property ShowProgress  : Boolean read fShowProgress write fShowProgress default True;
      property Title         : String  read fTitle write fTitle;
      property AfterPreview  : TNotifyEvent read fAfterPreview write fAfterPreview;
      property OnStartPage   : TNotifyEvent read fOnStartPage write fOnStartPage;
      property OnEndPage     : TNotifyEvent read fOnEndPage   write fOnEndPage;
      property OnFormatString: TZRFormatStringEvent read fOnFormatString write fOnFormatString;
  end;

  { TZCompositeReport - Component which combines several reports into one }
  TZCompositeReport = class(TComponent)
    private
      fOptions        : TZRPrinterOptions;
      fZRPrinter      : TZRPrinter;
      fReports        : TList;
      fShowProgress   : Boolean;
      fTitle          : String;
      fPageWidth      : Integer;
      fPageLength     : Integer;
      fOnAddReports   : TNotifyEvent;
      fReport         : Integer;  
      fOnFinished     : TNotifyEvent;
      fOnFormatString : TZRFormatStringEvent;
      procedure CreateComposite;
    public
      constructor Create(AOwner : TComponent); override;
      destructor Destroy; override;
      procedure Prepare;
      procedure Preview;
      procedure Print;
      property PageWidth : Integer read fPageWidth  write fPageWidth;
      property PageLength: Integer read fPageLength write fPageLength;
      property Report    : Integer read fReport;
      property Reports   : TList read fReports write fReports;
      property ZRPrinter : TZRPrinter read fZRPrinter write fZRPrinter;
    published
      property Options     : TZRPrinterOptions read fOptions write fOptions;
      property ShowProgress: Boolean read fShowProgress write fShowProgress default True;
      property Title       : String read fTitle write fTitle;
      property OnAddReports: TNotifyEvent read fOnAddReports write fOnAddReports;
      property OnFinished  : TNotifyEvent read fOnFinished   write fOnFinished;
      property OnFormatString: TZRFormatStringEvent read fOnFormatString write fOnFormatString;
  end;

const
  DefaultReportWidth  = 20;
  DefaultReportHeight = 10;

implementation

uses ZRConst, ZRStrUtl, ZRPrgres;

function DataSetOK(DataSet : TDataSet) : Boolean;
begin
  Result := (DataSet <> nil) and DataSet.Active;
end;

function ZRBandTypeName(BandType : TZRBandType) : string;
begin
  Result := LoadStr(SzrTitle + ord(BandType));
end;

function ZRBandComponentName(BandType : TZRBandType) : String;
begin
  Result := LoadStr(SzrTitleBandName + ord(BandType));
end;

{ TZRFrame }

constructor TZRFrame.Create(AOwner: TZReportControl);
begin
  inherited Create;
  FillChar(fValues, SizeOf(fValues), 0);
  fOwner:= aOwner;
end;

function TZRFrame.GetValue(Index: Integer): Integer;
begin
  case Index of
    4  : Result:= Right  + Left;
    5  : Result:= Bottom + Top;
    else Result:= fValues[Index];
  end;
end;
procedure TZRFrame.SetValue(Index: Integer; Value: Integer);
var
  DX, DY, i: Integer;
begin
  if fValues[Index] <> Value then
    begin
      if Index in [0,2] then DX:= Value - fValues[Index] else DX:= 0;
      if Index in [1,3] then DY:= Value - fValues[Index] else DY:= 0;
      fValues[Index]:= Value;
      if not (csLoading in fOwner.ComponentState) then
        begin
          fOwner.Width := fOwner.Width  + DX;
          fOwner.Height:= fOwner.Height + DY;
          if Index < 2 then
            for i:=0 to fOwner.ControlCount-1 do
              if (fOwner.Controls[i] is TZReportControl) then
                with (fOwner.Controls[i] as TZReportControl) do
                  begin
                    Left:= Left + DX;
                    Top := Top  + DY;
                  end;
        end;
    end;
end;

procedure TZRFrame.Paint;
var
  R  : TRect;
  dlt,
  drb: TPoint;
begin
  with fOwner.Canvas do
    begin
      R:= fOwner.ClientRect;
      Pen.Style:= psSolid;
      Pen.COlor:= clReportFrame;
      dlt:= fOwner.CharSize;    drb:= dlt;
      if Left  >0 then dlt.x:= dlt.x div 2 else dlt.x:= 0;
      if Top   >0 then dlt.y:= dlt.y div 2 else dlt.y:= 0;
      if Right >0 then drb.x:= drb.x div 2 else drb.x:= 0;
      if Bottom>0 then drb.y:= drb.y div 2 else drb.y:= 0;
      if Top>0 then begin
        MoveTo(dlt.x               , dlt.y);
        LineTo(R.Right-R.Left-drb.x, dlt.y);
      end;
      if Left>0 then begin
        MoveTo(dlt.x, dlt.y               );
        LineTo(dlt.x, R.Bottom-R.Top-drb.y);
      end;
      if Bottom>0 then begin
        MoveTo(dlt.x               , R.Bottom-R.Top-drb.y);
        LineTo(R.Right-R.Left-drb.x, R.Bottom-R.Top-drb.y);
      end;
      if Right>0 then begin
        MoveTo(R.Right-R.Left-drb.x, dlt.y               );
        LineTo(R.Right-R.Left-drb.x, R.Bottom-R.Top-drb.y);
      end;
    end;
end;

procedure TZRFrame.Print(X, Y: Integer);
const
  Styles: array[Boolean] of TEscapeStyles = ([], [esFrame]);
var
  S, HC: String;
  BY, i: Integer;
begin
  S:= Replicate(FrameChar[fsHorizontal], fOwner.Width-Width);
  if Left  > 0 then S:= FrameChar[fsCross] + Replicate(FrameChar[fsHorizontal], Left-1) + S;
  if Right > 0 then S:= S + Replicate(FrameChar[fsHorizontal], Right-1) + FrameChar[fsCross];
  if not (fOwner is TZRCustomBand) then
    begin
      Inc(X, fOwner.Left);
      Inc(Y, fOwner.Top );
    end;
  if fOwner.Length > fOwner.Height then
    BY:= Y + fOwner.Length-1
  else
    BY:= Y + fOwner.Height-1;

  if (Top   >0) then fOwner.PrintString(X, Y , S, Styles[fOwner.Report.ZRPrinter.Options.CorrectFrames]);
  if (Bottom>0) then fOwner.PrintString(X, BY, S, Styles[fOwner.Report.ZRPrinter.Options.CorrectFrames]);

  for i:=1 to fOwner.Length do
    begin
      if (i=1) and (Top>0) or (Y+i-1=BY) and (Bottom>0) then
        HC:= FrameChar[fsCross]
      else
        HC:= FrameChar[fsVertical];
      if (Left >0) then fOwner.PrintString(X               , Y+i-1, HC, Styles[fOwner.Report.ZRPrinter.Options.CorrectFrames]);
      if (Right>0) then fOwner.PrintString(X+fOwner.Width-1, Y+i-1, HC, Styles[fOwner.Report.ZRPrinter.Options.CorrectFrames]);
    end;
end;

{ TZReportControl }

constructor TZReportControl.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Color     := clReportControl;
  Width     := 8;
  Height    := 1;
  ParentFont:= True;
  fFrame    := TZRFrame.Create(Self);
end;

destructor TZReportControl.Destroy;
begin
  fFrame.Free;
  inherited Destroy;
end;

function TZReportControl.GetCharSize: TPoint;
var
  TM: TTextMetric;
begin
  if Assigned(Report) and Assigned(Report.Parent) and
     GetTextMetrics(Report.Canvas.Handle, TM) then
    with Result do
      begin
        X:= TM.tmAveCharWidth;
        Y:= TM.tmHeight;
      end
  else
    with Result do
      begin
        X:=  8;
        Y:= 16;
      end
end;

function TZReportControl.GetLeft: Integer;
begin
  Result:= inherited Left;
  if not (Self is TZReport) then Result:= Result div CharSize.X;
end;
procedure TZReportControl.SetLeft(Value: Integer);
begin
  if not (Self is TZReport) then
    inherited Left:= Value * CharSize.X
  else
    inherited Left:= Value;
end;

function TZReportControl.GetTop: Integer;
begin
  Result:= inherited Top;
  if not (Self is TZReport) then Result:= Result div CharSize.Y;
end;
procedure TZReportControl.SetTop(Value: Integer);
begin
  if not (Self is TZReport) then
    inherited Top:= Value * CharSize.Y
  else
    inherited Top:= Value;
end;

function TZReportControl.GetWidth: Integer;
begin
  Result:= inherited Width div CharSize.X;
  if Result = 0 then Inc(Result);
end;
procedure TZReportControl.SetWidth(Value: Integer);
begin
  inherited Width:= Value * CharSize.X + 1;
end;

function TZReportControl.GetHeight: Integer;
begin
  Result:= inherited Height div CharSize.Y;
//  if Result = 0 then Inc(Result);
end;
procedure TZReportControl.SetHeight(Value: Integer);
begin
  inherited Height:= Value * CharSize.Y + 1
end;

procedure TZReportControl.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  L, T, W, H: Integer;
begin
  if (csLoading in ComponentState) or fTextResize then
    inherited SetBounds(ALeft, ATop, AWidth, AHeight)
  else
    begin
      if not (Self is TZReport) then
        begin
          L := ALeft div CharSize.X;
          T := ATop  div CharSize.Y;
        end
      else
        begin
          L := ALeft;
          T := ATop;
        end;
      W := AWidth  div CharSize.X;
      H := AHeight div CharSize.Y;
      SetCharBounds(L, T, W, H);
    end;
end;

procedure TZReportControl.SetCharBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  fTextResize:= True;
  if not (Self is TZReport) and AlignToBand then
    begin
      case Alignment of
        taLeftJustify  :
          ALeft := TZRCustomBand(Parent).Frame.Left;
        taRightJustify :
          if Parent is TZRCustomBand then
            ALeft := TZRCustomBand(Parent).Width - AWidth -
                     TZRCustomBand(Parent).Frame.Right;
        taCenter       :
          if Parent is TZRCustomBand then
            ALeft := TZRCustomBand(Parent).Frame.Left +
                    (TZRCustomBand(Parent).FramedWidth - AWidth) div 2;
      end;
    end;
  if Assigned(Frame) and (AWidth  < Frame.Width ) then AWidth := Frame.Width;
  if Assigned(Frame) and (AHeight < Frame.Height) then AHeight:= Frame.Height;
  if Self is TZReport then
    SetBounds(ALeft, ATop, AWidth * CharSize.X, AHeight * CharSize.Y)
  else
    SetBounds(ALeft * CharSize.X, ATop * CharSize.Y, AWidth * CharSize.X, AHeight * CharSize.Y);
  Invalidate;
  fTextResize:= False;
end;

procedure TZReportControl.AdjustBounds;
begin
  SetCharBounds(Left, Top, Width, Height);
end;

procedure TZReportControl.CMFontChanged(var Message: TMessage);
begin
  inherited;
  Canvas.Font:= Self.Font;
  AdjustBounds;
end;

procedure TZReportControl.SetFontStyle(Value: TEscapeStyles);
begin
  if fFontStyle <> Value then
    begin
      fFontStyle:= Value;
      Invalidate;
    end;
end;

procedure TZReportControl.SetAlignment(Value: TAlignment);
begin
  if fAlignment <> Value then
    begin
      fAlignment := Value;
      AdjustBounds;
    end;
end;

procedure TZReportControl.SetAlignToBand(Value : Boolean);
begin
  if fAlignToBand <> Value then
    begin
      fAlignToBand:= Value;
      AdjustBounds;
    end;
end;

procedure TZReportControl.SetParent(AParent: TWinControl);
begin
  if (Self is TZReport) then
    inherited SetParent(AParent)
  else
    begin
      if (AParent <> nil) then
        begin
          if not (AParent is TZRCustomBand) then
            raise EInvalidOperation.CreateRes(SzrParentReport);
          if not (Self is TZRCustomBand) and (AParent is TZReport) then
            raise EInvalidOperation.CreateRes(SzrParentBand);
          fReport:= (AParent as TZReportControl).Report;
          if (Self is TZRCustomBand) then AParent:= fReport;
        end
      else
        fReport:= nil;
      inherited SetParent(AParent);
      if Assigned(Report) then AdjustBounds;
    end;
end;

procedure TZReportControl.Loaded;
begin
  inherited Loaded;
  AdjustBounds;
end;

procedure TZReportControl.DrawString(X, Y: Integer; S: String);
begin
  with Canvas do
    begin
      Font.Style:= [];
      if esBold      in FontStyle then Font.Style:= Font.Style + [fsBold];
      if esItalic    in FontStyle then Font.Style:= Font.Style + [fsItalic];
      if esUnderline in FontStyle then Font.Style:= Font.Style + [fsUnderline];
      TextOut(X * CharSize.X, Y * CharSize.Y, S);
    end;
end;

procedure TZReportControl.Paint;
var
  R: TRect;
begin
  R:= GetClientRect;
  with Canvas do
    begin
      Brush.Style:= bsSolid;
      Brush.Color:= Self.Color;
      Pen.Style:= psSolid;
      Pen.Color:= clReportFrame;
      Rectangle(0, 0, ClientWidth, ClientHeight);
    end;
  Frame.Paint;
end;

procedure TZReportControl.Prepare;
begin
  IsPrinting:= True;
end;

procedure TZReportControl.Unprepare;
begin
  IsPrinting:= False;
end;

procedure TZReportControl.DoBeforePrint(var DoPrint: Boolean);
begin
  fLength := FramedHeight;    
  if Assigned(BeforePrint) then BeforePrint(Self, DoPrint);
  if not (Self is TZRCustomBand) then Inc(fLength, Frame.Height);
  fDoPrint:= DoPrint;
end;

procedure TZReportControl.DoAfterPrint(Printed: Boolean);
begin
  if Printed then fFrame.Print(Report.CurrentX, Report.CurrentY);
  if Assigned(AfterPrint) then AfterPrint(Self, Printed);
  fDoPrint:= Enabled;
end;

function TZReportControl.PrintString(X, Y: Integer; S: String; Styles: TEscapeStyles): Boolean;
var
  Page: TZRPage;
begin
  Page:= Report.ZRPrinter.Page;
  Result:= (Y < Page.Length - Report.Margins.Bottom); //    
  if Result then Page[Y]:= EscapeStuff( EscapeFormat(S, Styles), Page[Y], Succ(X));
end;

function TZReportControl.GetFramed(Index: Integer): Integer;
begin
  case Index of
    0: Result:= Left + Frame.Left;
    1: Result:= Top  + Frame.Top;
    2: Result:= Width  - Frame.Width;
    3: Result:= Height - Frame.Height;
  end;
end;

procedure TZReportControl.PrintTo(ParentX, ParentY: Integer);
begin
end;

procedure TZReportControl.MasterDataAdvance;
begin
end;

{ TZRCustomBand }

constructor TZRCustomBand.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csAcceptsControls, csCaptureMouse, csClickEvents,
                   csSetCaption, csOpaque, csDoubleClicks, csReplicatable];
  Color    := clReportBand;
  Width    := 8;
  Height   := 2;
end;

destructor TZRCustomBand.Destroy;
begin
  if Report <> nil then Report.RemoveBand(Self);
  inherited Destroy;
end;

procedure TZRCustomBand.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  if Params.WndParent = 0 then Params.WndParent := Application.Handle;
end;

procedure TZRCustomBand.SetCharBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  i: Integer;
begin
  inherited SetCharBounds(ALeft, ATop, AWidth, AHeight);
  if not (Self is TZReport) and Assigned(Report) and
     not fParentResizing then Report.SetBandSizes;
  for i:= 0 to ControlCount-1 do
    if (Controls[i] is TZReportControl) then
      (Controls[i] as TZReportControl).AdjustBounds;
end;

procedure TZRCustomBand.PaintGrid;
var
  i,d,x,y: Integer;
begin
  with Canvas do
    begin
      Brush.Color:= clReportGrid;
      Brush.Style:= bsSolid;
      Pen  .Color:= clReportGrid;
      Pen  .Style:= psSolid;
      d:= CharSize.X;
      y:= CharSize.Y div 2;
      for i:= 1 to Width - 1 do
        begin
          x:= i*d;
          MoveTo(x, 1);
          LineTo(x, ClientRect.Bottom-1);
          if i mod 5 = 0 then
            begin
              Dec(x, CharSize.X div 2);
              Ellipse(x-1, y-1, x+1, y+1);
            end;
        end;
      d:= CharSize.Y;
      x:= CharSize.X div 2;
      for i:= 1 to Height - 1 do
        begin
          y:= i*d;
          MoveTo(1, y);
          LineTo(ClientRect.Right-1, y);
          if i mod 5 = 0 then
            begin
              Dec(y, CharSize.Y div 2);
              Ellipse(x-1, y-1, x+1, y+1);
            end;
        end;
    end;
end;

procedure TZRCustomBand.Paint;
begin
  inherited Paint;
  PaintGrid;
  if Report <> Self then
    with Canvas do
      begin
        Brush.Style:= bsClear;
        Font.Name  := 'Small Fonts';
        Font.Size  := 6;
        Font.Color := clWindowText;
        TextOut(2, ClientRect.Bottom - TextHeight('W') - 4, ZRBandTypeName(BandType));
      end;
end;

procedure TZRCustomBand.Prepare;
var
  i : Integer;
begin
  inherited Prepare;
  for i := 0 to ControlCount - 1 do
    if Controls[i] is TZReportControl then
      TZReportControl(Controls[i]).Prepare;
end;

procedure TZRCustomBand.Unprepare;
var
  i : Integer;
begin
  for i := 0 to ControlCount - 1 do
    if Controls[i] is TZReportControl then
      TZReportControl(Controls[i]).Unprepare;
  inherited Unprepare;
end;

procedure TZRCustomBand.DoBeforePrint(var DoPrint: Boolean);
var
  i : Integer;
  B : Boolean;
  C : TZReportControl;
begin
  inherited;
  if DoPrint and (Self <> Report) then
    for i := 0 to ControlCount - 1 do
      if Controls[i] is TZReportControl then
        begin
          C:= (Controls[i] as TZReportControl);
          B:= C.Enabled;
          C.DoBeforePrint(B);
          if B and (C.Length > Length) then Length:= C.Length;
        end;
  Inc(fLength, Frame.Height);
end;

procedure TZRCustomBand.DoAfterPrint(Printed: Boolean);
var
  i : Integer;
begin
  if (Self <> Report) then
    for i := 0 to ControlCount - 1 do
      if Controls[i] is TZReportControl then
        TZReportControl(Controls[i]).DoAfterPrint(Printed and TZReportControl(Controls[i]).fDoPrint);
  inherited;
end;

function TZRCustomBand.GetChild : TZRCustomBand;
var
  i : Integer;
begin
  Result := nil;
  if Report <> nil then
  begin
    for i := 0 to Report.BandList.Count - 1 do
    begin
      if (TZRCustomBand(Report.BandList[i]).BandType = rbChild) and
         ((TZRCustomBand(Report.BandList[i]).ParentBand) = Self) then
      begin
        Result := TZRCustomBand(Report.BandList[i]);
        Break;
      end
    end
  end
end;

function TZRCustomBand.GetHasChild : Boolean;
begin
  Result := (GetChild <> nil);
end;

procedure TZRCustomBand.SetHasChild(Value : Boolean);
var
  aBand      : TZRCustomBand;
  aName      : String;
  AComponent : TComponent;
begin
  if GetHasChild <> Value then
  begin
    if Value then
    begin
      if csDesigning in ComponentState then
      begin
        AComponent := Owner;
        while not (AComponent is TCustomForm) and (AComponent <> nil) do
          AComponent := AComponent.Owner;
        if AComponent <> nil then
        {$ifdef VER120}
          AName := IFormDesigner(TCustomForm(AComponent).Designer).
                   UniqueName(ZRBandComponentName(rbChild));
        {$else}
          AName := TFormDesigner(TCustomForm(AComponent).Designer).
                   UniqueName(ZRBandComponentName(rbChild));
        {$endif}
      end else
        AName := '';
      aBand := TZRCustomBand.Create(Owner);
      aBand.Name        := aName;
      aBand.Parent      := Parent;
      aBand.fParentBand := Self;
      aBand.BandType    := rbChild;
    end else
    begin
      aBand := GetChild;
      if aBand.HasChild then
        aBand.ChildBand.fParentBand := Self;
      aBand.Free;
    end
  end
end;

{procedure TZRCustomBand.SetParentBand(Value : TZRCustomBand);
var
  aBand : TZRCustomBand;
begin
  aBand := Value;
  while (aBand <> Self) and (aBand <> nil) do
    begin
      if aBand.BandType = rbChild then
        aBand := aBand.ParentBand
      else
        aBand := nil;
    end;
  if aBand = nil then
    begin
      fParentBand := Value;
      BandType    := rbChild;
      Report.RebuildBandList;
      Report.SetBandSizes;
    end
  else
    begin
      if csDesigning in ComponentState then
        ShowMessage(LoadStr(SzrNoCircular));
    end;
end;}

procedure TZRCustomBand.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = ParentBand) then
    if (ParentBand.BandType = rbChild) then
      fParentBand:= ParentBand.ParentBand
    else
      Free;      //ParentBand:= nil;
end;

procedure TZRCustomBand.SetBandType(Value : TZRBandType);
begin
  fBandType := Value;
  if Assigned(Report) then
    begin
      Report.RebuildBandList;
      Report.SetBandSizes;
      Invalidate;
    end
end;

procedure TZRCustomBand.SetParent(AParent: TWinControl);
begin
  if (Report <> nil) then Report.RemoveBand(Self);
  inherited SetParent(AParent);
  if (Parent is TZRCustomBand) and (Report <> nil) and (Report <> Self) then
    Report.AddBand(Self);
end;

procedure TZRCustomBand.RegisterBands;
begin
  if Assigned(Report) then Report.RegisterBand(Self);
end;

procedure TZRCustomBand.Print;
begin
  if Assigned(Report) then Report.PrintBand(Self);
end;

{ TZRGroup }

constructor TZRGroup.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  fEvaluator:= TZREvaluator.Create;
  BandType  := rbGroupHeader;
end;

destructor TZRGroup.Destroy;
begin
  fEvaluator.Free;
  inherited Destroy;
end;

procedure TZRGroup.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) then
    begin
      if AComponent = fFooterBand then
        fFooterBand := nil;
      if AComponent = fMaster then
        fMaster := nil;
    end
end;

procedure TZRGroup.SetExpression(Value : String);
begin
  fExpression := Value;
end;

procedure TZRGroup.SetFooterBand(Value : TZRCustomBand);
begin
  fFooterBand := Value;
  if fFooterBand <> nil then
    begin
      fFooterBand.BandType := rbGroupFooter;
      fFooterBand.FreeNotification(self);
    end;
end;

function TZRGroup.GetHasFooter: Boolean;
begin
  Result:= FooterBand <> nil;
end;

procedure TZRGroup.SetHasFooter(Value: Boolean);
var
  aBand      : TZRCustomBand;
  aName      : String;
  AComponent : TComponent;
begin
  if GetHasChild <> Value then
  begin
    if Value then
    begin
      if csDesigning in ComponentState then
      begin
        AComponent := Owner;
        while not (AComponent is TCustomForm) and (AComponent <> nil) do
          AComponent := AComponent.Owner;
        if AComponent <> nil then
        {$ifdef VER120}
          AName := IFormDesigner(TCustomForm(AComponent).Designer).
                   UniqueName(ZRBandComponentName(rbGroupFooter));
        {$else}
          AName := TFormDesigner(TCustomForm(AComponent).Designer).
                   UniqueName(ZRBandComponentName(rbGroupFooter));
        {$endif}
      end else
        AName := '';
      aBand := TZRBand.Create(Owner);
      aBand.Name       := aName;
      aBand.Parent     := Parent;
      FooterBand       := aBand;
    end else
    begin
      aBand := FooterBand;
      if aBand.HasChild then FooterBand:= aBand.ChildBand;
      aBand.Free;
    end
  end
end;

procedure TZRGroup.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (Master = nil) then SetMaster(Report);
end;

procedure TZRGroup.SetMaster(Value : TZRControllerBand);
begin
  if Value = nil then
    fMaster := Report
  else
    fMaster := Value;
  Report.RebuildBandList;
  Report.SetBandSizes;
  fMaster.FreeNotification(Self);
end;

procedure TZRGroup.Prepare;
begin
  fEvaluator.DataSets := Report.AllDataSets;
  fEvaluator.Prepare(Expression);
  HasResult := False;
  inherited;
end;

procedure TZRGroup.Unprepare;
begin
  inherited;
  fEvaluator.Unprepare;
end;

procedure TZRGroup.Check;
var
  aValue : TZREvValue;
begin
  Reprint := False;
  if not HasResult then
    begin
      GroupValue := fEvaluator.Value;
      Reprint    := True;
      HasResult  := True;
    end
  else
    begin
      aValue := fEvaluator.Value;
      if aValue.Kind <> GroupValue.Kind then
        Reprint := True
      else
      begin
        case aValue.Kind of
          valString  : Reprint := aValue.StrResult <> GroupValue.strResult;
          valInteger : Reprint := aValue.IntResult <> GroupValue.intResult;
          valDouble  : Reprint := aValue.dblResult <> GroupValue.dblResult;
          valBoolean : Reprint := aValue.booResult <> GroupValue.booResult;
          valDateTime: Reprint := aValue.datResult <> GroupValue.datResult;
        end;
      end;
      if Reprint then GroupValue := aValue;
    end;
end;

{ TZRControllerBand }

constructor TZRControllerBand.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  PrintBeforeList := TList.Create;
  PrintAfterList  := TList.Create;
  GroupList       := TList.Create;
  NotifyList      := TList.Create;
  Master          := nil;
  PrintBefore     := False;
  PrintIfEmpty    := False;
  BandType        := rbDetail;
  Detail          := Self;
  Header          := nil;
  Footer          := nil;
end;

destructor TZRControllerBand.Destroy;
begin
  inherited Destroy;
  NotifyList     .Free;
  GroupList      .Free;
  PrintAfterList .Free;
  PrintBeforeList.Free;
end;

procedure TZRControllerBand.SetDataSet(Value : TDataSet);
begin
  fDataSet := Value;
  if Value <> nil then
    fDataSet.FreeNotification(Self);
end;

procedure TZRControllerBand.SetMaster(Value : TZRControllerBand);
var
  aMaster: TZRControllerBand;
begin
  if (Value <> Self) and (Value is TZRControllerBand) then
    begin
      aMaster:= Value;
      while (aMaster <> Self) and (aMaster <> nil) do
         aMaster:= aMaster.Master;
      if aMaster = nil then
        begin
          fMaster:= Value;
          if (Report <> nil) and not (csDestroying in Report.ComponentState) then
            begin
              Report.RebuildBandList;
              Report.SetBandSizes;
            end;
          if Assigned(Value) then Value.FreeNotification(Self);
        end
      else
        begin
          if csDesigning in ComponentState then
            ShowMessage(LoadStr(SzrNoCircular));
        end;
    end;
end;

procedure TZRControllerBand.SetPrintBefore(Value : Boolean);
begin
  if PrintBefore <> Value then
    begin
      fPrintBefore := Value;
      if Report <> nil then
        begin
          Report.RebuildBandList;
          Report.SetBandSizes;
        end;
    end;
end;

procedure TZRControllerBand.SetParent(AParent: TWinControl);
begin
  inherited SetParent(AParent);
  if (Master = nil) and Assigned(Report) then SetMaster(Report);
end;

procedure TZRControllerBand.Notification(AComponent : TComponent; Operation : TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) then
    begin
      if AComponent = Header then
        Header := nil;
      if AComponent = Detail then
        Detail := nil;
      if AComponent = Footer then
        Footer := nil;
      if AComponent = DataSet then
        DataSet := nil;
      if AComponent = Master then
        Master := nil;
    end
end;

procedure TZRControllerBand.AddNotifyClient(Value : TZReportControl);
begin
  NotifyList.Add(Value);
end;

procedure TZRControllerBand.BuildTree;
var
  B: TZRControllerBand;
  i: Integer;
begin
  if (csDestroying in ComponentState) then Exit;
  PrintBeforeList.Clear;
  PrintAfterList .Clear;
  GroupList      .Clear;
  for i:= 0 to ControlCount-1 do
    begin
      if (csDestroying in Controls[i].ComponentState) then Continue;
      if Controls[i] is TZRControllerBand then
        begin
          B:= (Controls[i] as TZRControllerBand);
          if B.PrintBefore then
            B.Master.PrintBeforeList.Add(B)
          else
            B.Master.PrintAfterList.Add(B);
          B.BuildTree;
        end;
      if (Controls[i] is TZRGroup) then
        if (Controls[i] as TZRGroup).Master <> nil then
          (Controls[i] as TZRGroup).Master.GroupList.Add(Controls[i]);
    end;
end;

procedure TZRControllerBand.RegisterBands;
var
  I : Integer;
begin
  if Assigned(fHeader) then Header.RegisterBands;
  for I := 0 to GroupList.Count - 1 do
    TZRGroup(GroupList[I]).RegisterBands;
  for I := 0 to PrintBeforeList.Count - 1 do
    TZRCustomBand(PrintBeforeList[I]).RegisterBands;
  if Assigned(fDetail) then
    if (Detail <> Self) then
      Detail.RegisterBands
    else
      inherited RegisterBands;
  for I := 0 to PrintAfterList.Count - 1 do
    TZRCustomBand(PrintAfterList[I]).RegisterBands;
  for I := GroupList.Count - 1 downto 0 do
    if TZRGroup(GroupList[I]).FooterBand <> nil then
      TZRGroup(GroupList[I]).FooterBand.RegisterBands;
  if Assigned(fFooter) then Footer.RegisterBands;
end;

procedure TZRControllerBand.Print;
var
  MoreData : Boolean;
  RecCount : Integer;
  DSOK     : Boolean;

  procedure PrintGroupHeaders;
  var
    I : Integer;
  begin
    for I := 0 to GroupList.Count - 1 do
      if TZRGroup(GroupList[I]).Reprint then
        Report.PrintBand(TZRGroup(GroupList[I]));
  end;

  procedure PrintGroupFooters;
  var
    I : Integer;
  begin
    for I := GroupList.Count - 1 downto 0 do
      if TZRGroup(GroupList[I]).Reprint and
         (TZRGroup(GroupList[I]).FooterBand <> nil) then
        Report.PrintBand(TZRGroup(GroupList[I]).FooterBand);
  end;

  function CheckGroups : Boolean;
  var
    I, J : Integer;
  begin
    Result := False;
    for I := 0 to GroupList.Count - 1 do
    begin
      TZRGroup(GroupList[I]).Check;
      if TZRGroup(GroupList[I]).Reprint then
      begin
        Result := True;
        for J := I + 1 to GroupList.Count - 1 do
          with TZRGroup(GroupList[J]) do
            begin
              GroupValue := fEvaluator.Value;
              Reprint    := True;
              HasResult  := True;
            end;
        Exit;
      end;
    end;
  end;

  procedure CheckLastGroupFooters;
  var
    I : Integer;
  begin
    for I := 0 to GroupList.Count - 1 do
      TZRGroup(GroupList[I]).Reprint := not TZRGroup(GroupList[I]).Reprint;
  end;

  procedure PrintBeforeControllers;
  var
    I : Integer;
  begin
    for I := 0 to PrintBeforeList.Count - 1 do
      TZRControllerBand(PrintBeforeList[I]).Print;
  end;

  procedure PrintAfterControllers;
  var
    I : Integer;
  begin
    for I := 0 to PrintAfterList.Count - 1 do
      TZRControllerBand(PrintAfterList[I]).Print;
  end;

  procedure PrintEmptyController;
  begin
    if Assigned(Header) then Report.PrintBand(Header);
    PrintBeforeControllers;
    if Assigned(Detail) then Report.PrintBand(Detail);
    PrintAfterControllers;
    if Assigned(Footer) then Report.PrintBand(Footer);
  end;

begin
  RecCount := 1;
  if (DataSetOK(DataSet) or Assigned(fOnNeedData)) and
    Assigned(Report) then
  begin
    MoreData := True;
    DSOK     := DataSetOK(fDataSet);
    if DSOK then
      begin
        DataSet.First;
        MoreData := not DataSet.EOF;
        if Report.DataSet = DataSet then RecCount := Report.RecordCount;
        if (not Moredata) and PrintIfEmpty then
          PrintEmptyController;
      end
    else
      if Assigned(fOnNeedData) and not (csDesigning in ComponentState) then
        OnNeedData(Self, MoreData);
    fDetailNumber := 0;
    CheckGroups;
    if MoreData then
      begin
        Application.ProcessMessages;
        if Report.ZRPrinter.Cancelled then Exit;
        if Assigned(Header) then Report.PrintBand(fHeader);
        if (Self = Report) and (Report.PageNumber = 1) then
          Report.PrintBand(Report.Bands.ColumnHeaderBand);
        while MoreData do
          begin
            Application.ProcessMessages;
            if Report.ZRPrinter.Cancelled then Exit;
            Inc(fDetailNumber);
            PrintGroupHeaders;
            PrintBeforeControllers;
            MasterDataAdvance;
            if Assigned(Detail) then
              if (Detail <> Self) then
                Report.PrintBand(fDetail)
              else
                inherited Print;
            PrintAfterControllers;
            if DSOK then
              begin
                DataSet.Next;
                MoreData := not fDataSet.EOF;
              end
            else
              begin
                MoreData := False;
                if Assigned(fOnNeedData) and not (csDesigning in ComponentState) then
                  OnNeedData(Self, MoreData);
              end;
            if CheckGroups then
              begin
                if DSOK then DataSet.Prior;
                PrintGroupFooters;
                if DSOK then DataSet.Next;
              end;
            if DSOK and (Report.DataSet = DataSet) and (RecCount <> 0) then
              Report.ZRPrinter.Progress := (Longint(DetailNumber) * 100) div RecCount;
          end;
      CheckLastGroupFooters;
      PrintGroupFooters;
      if (Self = Report) then Report.PrintBand(Report.Bands.ColumnFooterBand);
      if Assigned(Footer) then Report.PrintBand(Footer);
    end
  end
  else
    if PrintIfEmpty then
      PrintEmptyController;
end;

procedure TZRControllerBand.Prepare;
var
  I : Integer;
begin
  if Self = Report then BuildTree;
  NotifyList.Clear;
  if DataSetOK(fDataSet) then
    Report.AllDataSets.Add(DataSet);
  for I := 0 to PrintBeforeList.Count - 1 do
    TZRControllerBand(PrintBeforeList[I]).Prepare;
  for I := 0 to PrintAfterList.Count - 1 do
    TZRControllerBand(PrintAfterList[I]).Prepare;
  for I := 0 to GroupList.Count - 1 do
    TZRGroup(GroupList[I]).Prepare;
  inherited;
end;

procedure TZRControllerBand.Unprepare;
var
  I : Integer;
begin
  for I := 0 to PrintBeforeList.Count - 1 do
    TZRControllerBand(PrintBeforeList[I]).Unprepare;
  for I := 0 to PrintAfterList.Count - 1 do
    TZRControllerBand(PrintAfterList[I]).Unprepare;
  for I := 0 to GroupList.Count - 1 do
    TZRGroup(GroupList[I]).Unprepare;
  inherited;
end;

procedure TZRControllerBand.MasterDataAdvance;
var
  i: Integer;
begin
  for i:=0 to NotifyList.Count-1 do
    TZReportControl(NotifyList[i]).MasterDataAdvance;
end;

{ TZRSubDetailBand }

constructor TZRSubDetailBand.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  Bands    := TZRSubDetailBands.Create(Self);
  BandType := rbSubDetail;
end;

destructor TZRSubDetailBand.Destroy;
begin
  inherited Destroy;
  Bands.Free;
end;

function TZRSubDetailBand.GetFooterBand : TZRCustomBand;
begin
  Result := Footer;
end;

function TZRSubDetailBand.GetHeaderBand : TZRCustomBand;
begin
  Result := Header;
end;

procedure TZRSubDetailBand.SetFooterBand(Value : TZRCustomBand);
begin
  Footer := Value;
  if Footer <> nil then
    begin
      Footer.BandType := rbGroupFooter;
      Footer.FreeNotification(Self);
    end
end;

procedure TZRSubDetailBand.SetHeaderBand(Value : TZRCustomBand);
begin
  Header := Value;
  if Header <> nil then
    begin
      Header.BandType := rbGroupHeader;
      Header.FreeNotification(Self);
    end
end;

{ TZReport }

constructor TZReport.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fMargins       := TZRMargins.Create(Self);
  fAllDataSets   := TList.Create;
  fBandList      := TList.Create;
  BandRegList    := TList.Create;
  fReport        := Self;
  fBands         := TZReportBands.Create(Self);
  fColumns       := 1;
  Detail         := nil;
  fOptions       := TZRPrinterOptions.Create(Self);
  Color          := clReport;
  Font.Name      := DefaultFontName;
  Width          := DefaultReportWidth;
  Height         := DefaultReportHeight;
  fReady         := True;
  fShowProgress  := True;
end;

destructor TZReport.Destroy;
begin
  inherited Destroy;
  fMargins       .Free;
  fBands         .Free;
  BandRegList    .Free;
  fBandList      .Free;
  fAllDataSets   .Free;
  fOptions       .Free;
end;

procedure TZReport.Loaded;
begin
  inherited Loaded;
  RebuildBandList;
  AdjustBounds;
end;

procedure TZReport.Paint;
  procedure PaintMargins;
  begin
    with Canvas do
      begin
        Brush.Style:= bsClear;
        Pen.Style  := psSolid;
        Pen.Color  := clReportFrame;
        Rectangle(
          Margins.Left   * CharSize.X -1,
          Margins.Top    * CharSize.Y -1,
          (Width -Margins.Right ) * CharSize.X,
          (Height-Margins.Bottom) * CharSize.Y );
      end;
  end;
  procedure PaintColumns;
  var
    i           : Integer;
  begin
    if Columns > 1 then
    with Canvas do
      begin
        Brush.Style:= bsClear;
        Pen.Style  := psDash;
        Pen.Color  := clReportFrame;
        Rectangle(
          Margins.Left   * CharSize.X -1,
          Margins.Top    * CharSize.Y -1,
          (Width -Margins.Right ) * CharSize.X,
          (Height-Margins.Bottom) * CharSize.Y );
        Pen.Color  := clRed;
        for i := 1 to Columns-1 do
          begin
            MoveTo(CharSize.X * (Margins.Left + i * ColumnWidth + (i - 1) * ColumnSpace), 0);
            LineTo(CharSize.X * (Margins.Left + i * ColumnWidth + (i - 1) * ColumnSpace),
                   CharSize.Y * Margins.Top);

            MoveTo(CharSize.X * (Margins.Left + i * (ColumnWidth + ColumnSpace)), 0);
            LineTo(CharSize.X * (Margins.Left + i * (ColumnWidth + ColumnSpace)),
                   CharSize.Y * Margins.Top);
          end;
      end;
  end;
begin
  inherited Paint;
  PaintMargins;
  PaintColumns;
end;

function TZReport.CreateBand(BandType : TZRBandType) : TZRBand;
var
  {$ifdef VER120}
  Designer : IFormDesigner;
  {$else}
  Designer : TFormDesigner;
  {$endif}
  BandOwner: TComponent;
  aName : String;
  aBand : TZRBand;

  {$ifdef VER120}
  function FindDesigner: IFormDesigner;
  {$else}
  function FindDesigner: TFormDesigner;
  {$endif}
  var
    Component: TComponent;
  begin
    Result := nil;
    if csDesigning in ComponentState then
    begin
      Component := Owner;
      while not (Component is TCustomForm) and
        (Component <> nil) do
        Component := Component.Owner;
      {$ifdef VER120}
      Result := IFormDesigner(TCustomForm(Component).Designer);
      {$else}
      Result := TFormDesigner(TCustomForm(Component).Designer);
      {$endif}
    end
  end;

begin
  Designer := FindDesigner;
  BandOwner := nil;
  if Designer <> nil then BandOwner := Designer.GetRoot;
  if BandOwner = nil then BandOwner := Owner;
  if BandOwner = nil then BandOwner := Self;
  aBand  := TZRBand.Create(BandOwner);
  Result := aBand;
  Result.Parent   := Self;
  Result.BandType := BandType;
  if Designer <> nil then
    begin
      aName := ZRBandComponentName(BandType);
      if Copy(aName, 1, 1) = 'T' then aName := 'T'+ aName;
      Result.Name := Designer.UniqueName(aName);
      Designer.Modified;
    end;
end;

procedure TZReport.AddBand(aBand: TZRCustomBand);
begin
  if fBandList.IndexOf(aBand) < 0 then
    begin
      fBandList.Add(aBand);
      RebuildBandList;
      SetBandSizes;
    end;
end;

procedure TZReport.RemoveBand(aBand: TZRCustomBand);
begin
  if not (csDestroying in ComponentState) then
    begin
      fBandList.Remove(aBand);
      RebuildBandList;
      SetBandSizes;
    end;
end;

procedure TZReport.RegisterBand(aBand : TZRCustomBand);
begin
  if not (csDestroying in aBand.ComponentState) and
    (BandRegList.IndexOf(ABand) < 0) then
  begin
    BandRegList.Add(ABand);
    if aBand.HasChild then aBand.ChildBand.RegisterBands;
  end;
end;

procedure TZReport.RegisterBands;

  procedure AddBandsToList(BandType: TZRBandType);
  var
    i: Integer;
  begin
    for i:= 0 to fBandList.Count - 1 do
      if TZRCustomBand(fBandList[i]).BandType = BandType then
        TZRCustomBand(fBandList[i]).RegisterBands;
  end;

var
  i: Integer;
begin
  AddBandsToList(rbPageHeader);
  if Bands.HasTitle then AddBandsToList(rbTitle);
  AddBandsToList(rbColumnHeader);
  inherited RegisterBands;
  AddBandsToList(rbColumnFooter);
  if Bands.HasSummary then AddBandsToList(rbSummary);
  AddBandsToList(rbPageFooter);
  for i:= 0 to fBandList.Count - 1 do
    if BandRegList.IndexOf(fBandList[i]) = -1 then
      TZRCustomBand(fBandList[i]).RegisterBands;
end;

procedure TZReport.RebuildBandList;
begin
  if not (csLoading in ComponentState) then
    begin
      Detail:= Bands.DetailBand;
      Header:= Bands.TitleBand;
      Footer:= Bands.SummaryBand;
      BuildTree;
      BandRegList.Clear;
      RegisterBands;
      fBandList.Free;
      fBandList  := BandRegList;
      BandRegList:= TList.Create;
    end;
end;

procedure TZReport.SetCharBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  inherited SetCharBounds(ALeft, ATop, AWidth, AHeight);
  SetBandSizes;
end;

const
  ImplicitBands   = [rbPageHeader, rbPageFooter, rbTitle];
  ReportWideBands = [rbPageHeader, rbPageFooter, rbTitle];
  FooterBands     = [rbPageFooter, rbColumnFooter];

procedure TZReport.SetBandSizes;
var
  i, y,
  BandWidth  : Integer;
  aBand      : TZRCustomBand;
  aBandType  : TZRBandType;
  aChildBand : TZRCustomBand;
begin
  if not Assigned(fBandList) or
    (csDestroying in ComponentState) or
    (csLoading in ComponentState) then Exit;
  y          := Margins.Top;
  BandWidth  := Width - Margins.Left - Margins.Right;
  for i:= 0 to fBandList.Count - 1 do
    begin
      aBand:= TZRCustomBand(fBandList[i]);
      if Assigned(aBand) and not (csDestroying in aBand.ComponentState) then
        begin
          aBand.fParentResizing:= True;
          aBand.Left:= Margins.Left;
          aBand.Top := y;
          Inc(y, aBand.Height);
          aBandType:= aBand.BandType;
          if aBand is TZRCustomBand then
            begin
              aChildBand:= TZRCustomBand(aBand);
              while (aChildBand.ParentBand <> nil) and
                    (aChildBand.ParentBand.BandType = rbChild) do
                aChildBand:= TZRCustomBand(aChildBand.ParentBand);
              if aChildBand.ParentBand <> nil then
                aBandType:= aChildBand.ParentBand.BandType;
            end;
          if aBandType in ReportWideBands then
            aBand.Width:= BandWidth
          else
            aBand.Width:= ColumnWidth;
          aBand.fParentResizing:= False;
        end;
    end;
end;

procedure TZReport.SetColumns(Value: Integer);
begin
  if Columns <> Value then
    begin
      fColumns:= Value;
      SetBandSizes;
      Invalidate;
    end;
end;

procedure TZReport.SetColumnSpace(Value: Integer);
begin
  if ColumnSpace <> Value then
    begin
      fColumnSpace:= Value;
      SetBandSizes;
      Invalidate;
    end;
end;

function TZReport.ColumnWidth: Integer;
begin
  Result := (Width - Margins.Left - Margins.Right -
            ColumnSpace * (Columns - 1)) div Columns;
end;

function TZReport.GetRecordCount : Integer;
begin
  Result := 1;
  if DataSetOK(DataSet) then
    begin
      try
        Result := DataSet.RecordCount;
      except
        Result := 0;
      end;
    end;
end;

function TZReport.PageFooterSize(Footers: TZRBandTypes): Integer;
var
  I     : Integer;
  aBand : TZRCustomBand;
begin
  Result := 0;
  for I := 0 to ControlCount - 1 do
    if Controls[I] is TZRCustomBand then
      with TZRCustomBand(Controls[I]) do
        if (BandType in Footers) and Enabled then
          begin
            Inc(Result, Height);
            aBand := TZRCustomBand(Self.Controls[I]);
            while aBand.HasChild do
              begin
                aBand := aBand.ChildBand;
                Inc(Result, aBand.Height);
              end;
          end;
end;

function TZReport.RemainSpace : Integer;
begin
  if not Ready then
    Result := ZRPrinter.PageLength - Margins.Bottom - PageFooterSize(FooterBands) - CurrentY
  else
    Result := 0;
end;

procedure TZReport.ForceNewColumn;
begin
  if (not Ready) then
    begin
      if NewColumnForced or (CurrentColumn > 1) then
        NewColumn
      else
        NewColumnForced := True;
    end;
end;

procedure TZReport.ForceNewPage;
begin
  if not Ready then
    begin
      if NewPageForced or (PageNumber > 1) then
      begin
        repeat
          NewColumn
        until CurrentColumn = 1
      end else
        NewPageForced := True;
    end;
end;

procedure TZReport.NewColumn;
begin
  EndColumn;
  if fCurrentColumn = Columns then
    begin
      fCurrentColumn := 1;
      fCurrentX      := Margins.Left;
      NewPage;
    end
  else
    begin
      fCurrentX := fCurrentX + ColumnWidth + ColumnSpace;
      fCurrentY := fColumnTop;
      Inc(fCurrentColumn);
    end;
  if Bands.ColumnHeaderBand <> nil then Bands.ColumnHeaderBand.Print;
end;

procedure TZReport.NewPage;
begin
  EndPage;
  Inc(fPageCount);
  ZRPrinter.BeginPage;
  CurrentY := Margins.Top;
  if Assigned(fOnStartPage) and not (csDesigning in ComponentState) then fOnStartPage(Self);
  if (fPageCount > 1) or ((fPageCount = 1) and Options.FirstPageHeader) then
    if Bands.PageHeaderBand <> nil then Bands.PageHeaderBand.Print;
  fColumnTop:= CurrentY;
  if (PageNumber = 1) and Bands.HasTitle then Inc(fColumnTop, Bands.TitleBand.Height);
end;

procedure TZReport.EndColumn;
begin
  if Bands.ColumnFooterBand <> nil then Bands.ColumnFooterBand.Print;
end;

procedure TZReport.EndPage;
begin
  if fPageCount > 0 then
    begin
      if Assigned(fOnEndPage) and not (csDesigning in ComponentState) then fOnEndPage(Self);
      CurrentY := ZRPrinter.PageLength - Margins.Bottom - PageFooterSize([rbPageFooter]);
      if (Bands.PageFooterBand <> nil) and (fPageCount > 0) and
         ((not fLastPage) or Options.LastPageFooter) then Bands.PageFooterBand.Print;
    end;
end;

procedure TZReport.PrintBand(aBand : TZRCustomBand);
var
  PrintBand: Boolean;
  i        : Integer;
begin
  if not Assigned(aBand) or not aBand.Enabled or (csDesigning in ComponentState) then Exit;
//  if aBand.AlignToBottom then
//    CurrentY:= ZRPrinter.PageLength - Margins.Bottom - aBand.Height - PageFooterSize;
  with aBand do
    begin
      PrintBand:= Enabled;
      DoBeforePrint(PrintBand);
      if PrintBand then
        begin
          if ForceNewPage   and not (BandType in ImplicitBands) then Report.ForceNewPage   else
          if ForceNewColumn and not (BandType in ImplicitBands) then Report.ForceNewColumn else
          if not (BandType in FooterBands) and (Report.RemainSpace < Length) then Report.NewColumn;
          for i := 0 to ControlCount - 1 do
            if Controls[I] is TZReportControl and
               TZReportControl(Controls[I]).Enabled and
               TZReportControl(Controls[I]).fDoPrint then
              TZReportControl(Controls[I]).PrintTo(CurrentX, CurrentY);
          DoAfterPrint(PrintBand);
          CurrentY := CurrentY + Length;
        end;
      if HasChild then
        begin
          if BandType in FooterBands then ChildBand.BandType:= BandType;
          ChildBand.Print;
          ChildBand.BandType:= rbChild;
        end;
    end;
end;

procedure TZReport.CreateReport(CompositeReport : Boolean);
var
  I           : Integer;
  PrintReport : Boolean;
begin
  if Ready then
    begin
      fReady          := False;
      fLastPage       := False;
      NewColumnForced := False;
      NewPageForced   := False;
      PrintReport     := Enabled;
      try
        if Parent is TCustomForm then
          for I := 0 to TCustomForm(Parent).ComponentCount - 1 do
            if TCustomForm(Parent).Components[I] is TDataset then
              fAllDatasets.Add(TDataset(TCustomForm(Parent).Components[I]));
        DoBeforePrint(PrintReport);
        if PrintReport then
        try
          if not CompositeReport then
            begin
              ZRPrinter.BeginDoc;
              fCurrentY := ZRPrinter.PageLength;
              fPageCount:= 0;
            end;
          fCurrentColumn := 1;
          fCurrentX := Margins.Left;
          Detail := Bands.DetailBand;
          Header := Bands.TitleBand;
          Footer := Bands.SummaryBand;
          Prepare;
          if not CompositeReport then NewPage;
          inherited Print;
          if not CompositeReport then
            begin
              fCurrentX := Margins.Left;
              fLastPage := True;
              if not ZRPrinter.Cancelled then EndPage;
            end;
          Unprepare;
        finally
          if not CompositeReport and not ZRPrinter.Cancelled then ZRPrinter.EndDoc;
        end;
      finally
        fAllDataSets.Clear;
      end;
      fReady := True;
    end;
end;

function PrepareZRPrinter(Owner: TComponent): Boolean;
var
  aPrinter : TZRPrinter;
  aReceiver: TWinControl;
begin
  Result:= not (Owner is TZReport) or (Owner as TZReport).Ready;
  if Result then
    begin
      if (Owner is TZCompositeReport) then
        aPrinter:= (Owner as TZCompositeReport).fZRPrinter
      else
        aPrinter:= (Owner as TZReport).fZRPrinter;
      if Assigned(aPrinter) then
        begin
          AReceiver := aPrinter.MessageReceiver;
          aPrinter.Free;
        end
      else
        AReceiver := nil;
      aPrinter := TZRPrinter.Create(Owner);
      if (Owner is TZCompositeReport) then
        with (Owner as TZCompositeReport) do
          begin
            fZRPrinter    := aPrinter;
            aPrinter.OnFormatString:= OnFormatString;
          end
      else
        with (Owner as TZReport) do
          begin
            fZRPrinter:= aPrinter;
            ZRPrinter.AfterPreview  := PreviewFinished;
            ZRPrinter.AfterPrint    := PrintFinished;
            aPrinter .OnFormatString:= OnFormatString;
          end;
      aPrinter.MessageReceiver := AReceiver;
    end;
end;

procedure TZReport.Print;
var
  AProgress : TZRProgressForm;
begin
  AProgress := nil;
  if PrepareZRPrinter(Self) then
  try
//    ZRPrinter.Destination := zrdPrinter;
    AProgress := TZRProgressForm.Create(Application);
    AProgress.ZRPrinter := ZRPrinter;
    if ShowProgress then AProgress.Show;
    ZRPrinter.MessageReceiver := AProgress;
    CreateReport(False);
    ZRPrinter.Print;
  finally
    ZRPrinter.Free;
    ZRPrinter := nil;
    AProgress.Free;
  end;
end;

procedure TZReport.Preview;
var
  aZRPrinter : TZRPrinter;
begin
  aZRPrinter := nil;
  if PrepareZRPrinter(Self) and not ZRPrinter.ShowingPreview then
  try
    ZRPrinter.Preview;
    Application.ProcessMessages;
    CreateReport(False);
    aZRPrinter := ZRPrinter;
    repeat
      Application.ProcessMessages
    until (not aZRPrinter.ShowingPreview) or Application.Terminated;
  finally
    aZRPrinter.Free;
    ZRPrinter := nil;
  end;
end;

procedure TZReport.PreviewFinished(Sender : TObject);
begin
  if Assigned(fAfterPreview) then fAfterPreview(Self);
end;

procedure TZReport.PrintFinished(Sender : TObject);
begin
  DoAfterPrint(True);
end;

{ TZCompositeReport }

constructor TZCompositeReport.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  fReports     := TList.Create;
  fPageWidth   := DefaultReportWidth;
  fPageLength  := DefaultReportHeight;
  fOptions     := TZRPrinterOptions.Create(Self);
  fShowProgress:= True;
end;

destructor TZCompositeReport.Destroy;
begin
  fOptions.Free;
  fReports.Free;
  inherited Destroy;
end;

procedure TZCompositeReport.CreateComposite;
var
  I            : Integer;
  LastCurrentY : Integer;
  LastPageCount: Integer;
  LastColumn   : Integer;
  aReport      : TZReport;
begin
  Reports.Clear;
  if Assigned(fOnAddReports) then fOnAddReports(Self);
  if Reports.Count > 0 then
    begin
      fZRPrinter.BeginDoc;
      fZRPrinter.BeginPage;
      aReport := TZReport(Reports[0]);
      LastCurrentY  := aReport.Margins.Top;
      LastPageCount := 1;
      LastColumn    := 1;
      for i := 0 to Reports.Count - 1 do
        begin
          fReport := i;
          aReport := TZReport(Reports[fReport]);
          aReport.ZRPrinter      := fZRPrinter;
          aReport.CurrentY       := LastCurrentY;
          aReport.fPageCount     := LastPageCount;
          aReport.fCurrentColumn := LastColumn;
          aReport.CreateReport(True);
          if Assigned(fOnFinished) then fOnFinished(Reports[fReport]);
          LastCurrentY  := aReport.CurrentY;
          LastPageCount := aReport.fPageCount;
          LastColumn    := aReport.CurrentColumn;
        end;
      fZRPrinter.EndDoc;
    end;
  if Assigned(fOnFinished) then fOnFinished(Self);
end;

procedure TZCompositeReport.Print;
var
  AProgress : TZRProgressForm;
begin
  AProgress := nil;
  if PrepareZRPrinter(Self) then
  try
//  fOptions.Destination := zrdPrinter;
    AProgress := TZRProgressForm.Create(Application);
    AProgress.ZRPrinter := fZRPrinter;
    if ShowProgress then AProgress.Show;
    fZRPrinter.MessageReceiver := AProgress;
    CreateComposite;
    ZRPrinter.Print;
  finally
    fZRPrinter.Free;
    fZRPrinter := nil;
    AProgress.Free;
  end;
end;

procedure TZCompositeReport.Preview;
begin
  if PrepareZRPrinter(Self) then
    try
      ZRPrinter.Preview;
      CreateComposite;
      repeat
        Application.ProcessMessages
      until (not ZRPrinter.ShowingPreview) or Application.Terminated;
    finally
      ZRPrinter.Free;
      ZRPrinter := nil;
    end;
end;

procedure TZCompositeReport.Prepare;
begin
  fZRPrinter := TZRPrinter.Create(Self);
  CreateComposite;
end;

{ TZRSubDetailBands }

constructor TZRSubDetailBands.Create(AOwner : TZRSubDetailBand);
begin
  inherited Create;
  Owner := AOwner;
end;

function TZRSubDetailBands.GetHasFooter : Boolean;
begin
  Result := Owner.FooterBand <> nil;
end;

function TZRSubDetailBands.GetHasHeader : Boolean;
begin
  Result := Owner.HeaderBand <> nil;
end;

function TZRSubDetailBands.GetHeaderBand : TZRCustomBand;
begin
  Result := Owner.HeaderBand;
end;

function TZRSubDetailBands.GetFooterBand : TZRCustomBand;
begin
  Result := Owner.FooterBand;
end;

procedure TZRsubDetailBands.SetHasHeader(Value : Boolean);
begin
  if Value then
    begin
      if not HasHeader then
        Owner.HeaderBand := Owner.Report.CreateBand(rbGroupHeader)
    end
  else
    begin
      if HasHeader then
        begin
          HeaderBand.Free;
          Owner.HeaderBand := nil;
        end;
    end;
end;

procedure TZRSubDetailBands.SetHasFooter(Value : Boolean);
begin
  if Value then
    begin
      if not HasFooter then
        Owner.FooterBand := Owner.Report.CreateBand(rbGroupFooter);
    end
  else
    begin
      if HasFooter then
        begin
          FooterBand.Free;
          Owner.FooterBand := nil;
        end;
    end
end;

{ TZReportBands }

constructor TZReportBands.Create(AOwner : TZReport);
begin
  inherited Create;
  fOwner := AOwner;
end;

function TZReportBands.BandInList(BandType : TZRBandType) : TZRCustomBand;
var
  I : Integer;
begin
  Result := nil;
  if Assigned(fOwner) and (fOwner.BandList <> nil) then
    begin
      for I := 0 to fOwner.BandList.Count - 1 do
        begin
          if TZRCustomBand(fOwner.BandList[I]).BandType = BandType then
            begin
              Result := TZRCustomBand(fOwner.BandList[I]);
              Break;
            end
        end
    end
end;

function TZReportBands.GetHasBand(Index : Integer) : Boolean;
begin
  case Index of
    1 : Result := BandInList(rbTitle       ) <> nil;
    2 : Result := BandInList(rbPageHeader  ) <> nil;
    3 : Result := BandInList(rbColumnHeader) <> nil;
    4 : Result := BandInList(rbDetail      ) <> nil;
    5 : Result := BandInList(rbColumnFooter) <> nil;
    6 : Result := BandInList(rbPageFooter  ) <> nil;
  else
    Result := BandInList(rbSummary) <> nil;
  end
end;

function TZReportBands.GetBand(Index : Integer) : TZRCustomBand;
begin
  case Index of
    1 : Result := BandInList(rbTitle       );
    2 : Result := BandInList(rbPageHeader  );
    3 : Result := BandInList(rbColumnHeader);
    4 : Result := BandInList(rbDetail      );
    5 : Result := BandInList(rbColumnFooter);
    6 : Result := BandInList(rbPageFooter  );
  else
    Result := BandInList(rbSummary);
  end;
end;

procedure TZReportBands.SetBand(BandType : TZRBandType; Value : Boolean);
var
  aBand : TZRCustomBand;
begin
  if (BandInList(BandType) <> nil) <> Value then
    begin
      if Value then
        TZReport(fOwner).CreateBand(BandType)
      else
      begin
        aBand := BandInList(BandType);
        aBand.Free;
      end
    end
end;

procedure TZReportBands.SetHasBand(Index : Integer; Value : Boolean);
begin
  case Index of
    1 : SetBand(rbTitle       , Value);
    2 : SetBand(rbPageHeader  , Value);
    3 : SetBand(rbColumnHeader, Value);
    4 : SetBand(rbDetail      , Value);
    5 : SetBand(rbColumnFooter, Value);
    6 : SetBand(rbPageFooter  , Value);
    7 : SetBand(rbSummary     , Value);
  end;
end;

{ TZRMargins }
constructor TZRMargins.Create(AOwner: TZReport);
begin
  inherited Create;
  fOwner  := AOwner;
  fMargins:= Rect(2, 1, 1, 1);
  Changed;
end;

function TZRMargins.GetValue(Index: Integer): Integer;
begin
  case Index of
    0: Result:= fMargins.Left;
    1: Result:= fMargins.Top;
    2: Result:= fMargins.Right;
    3: Result:= fMargins.Bottom;
  end;
end;

procedure TZRMargins.SetValue(Index: Integer; Value: Integer);
begin
  case Index of
    0: fMargins.Left   := Value;
    1: fMargins.Top    := Value;
    2: fMargins.Right  := Value;
    3: fMargins.Bottom := Value;
  end;
  Changed;
end;

procedure TZRMargins.Changed;
begin
  fOwner.SetBandSizes;
  fOwner.Invalidate;
end;

end.

(*
  { TZRCreateReportThread - class used to spawn background thread }
  TZRCreateReportThread = class(TThread)
    private
      fZRPrinter : TZRPrinter;
      fReport    : TZReport;
    public
      constructor Create(AReport : TZReport);
      procedure Execute; override;
      property Report : TZReport read fReport write fReport;
  end;

{ TZRCreateReportTread - class for background preparing/printing. 32 bit only }

constructor TZRCreateReportThread.Create(AReport : TZReport);
begin
  fReport        := AReport;
  FreeOnTerminate:= True;
  fZRPrinter     := AReport.ZRPrinter;
  inherited Create(False);
end;

procedure TZRCreateReportThread.Execute;
begin
  fReport.CreateReport(False);
  fZRPrinter.Free;
end;
*)

