unit ZRPrev;

interface

uses Windows, SysUtils, Classes, Controls, Forms, Graphics, Dialogs,
     StdCtrls, Buttons, ExtCtrls, ComCtrls, ZRPrntr;

type
  TZRPreviewForm = class(TForm)
    Toolbar: TPanel;
    StatusPanel: TPanel;
    FirstPage: TSpeedButton;
    PrevPage: TSpeedButton;
    NextPage: TSpeedButton;
    LastPage: TSpeedButton;
    Setup: TSpeedButton;
    Print: TSpeedButton;
    Save: TSpeedButton;
    Load: TSpeedButton;
    InfoPanel: TPanel;
    Status: TLabel;
    ProgressBar: TProgressBar;
    ZRPreview: TZRPreview;
    QuitPanel: TPanel;
    Exit: TBitBtn;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FirstPageClick(Sender: TObject);
    procedure PrevPageClick(Sender: TObject);
    procedure NextPageClick(Sender: TObject);
    procedure LastPageClick(Sender: TObject);
    procedure PrintClick(Sender: TObject);
    procedure ExitClick(Sender: TObject);
    procedure SaveClick(Sender: TObject);
    procedure SetupClick(Sender: TObject);
    procedure LoadClick(Sender: TObject);
    procedure ZRPreviewPageAvailable(Sender: TObject; PageNum: Integer);
    procedure ZRPreviewProgressUpdate(Sender: TObject; Progress: Integer);
    procedure FormCreate(Sender: TObject);
    procedure ZRPreviewKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    function  GetPrinter: TZRPrinter;
    procedure SetPrinter(Value: TZRPrinter);
  public
    constructor CreatePreview(AOwner : TComponent; aZRPrinter : TZRPrinter); virtual;
    procedure UpdateInfo;
    property ZRPrinter : TZRPrinter read GetPrinter write SetPrinter;
  end;

implementation

{$R *.DFM}

uses ZRConst;

{ TZRPreviewForm }

constructor TZRPreviewForm.CreatePreview(AOwner : TComponent; aZRPrinter : TZRPrinter);
begin
  inherited Create(AOwner);
  ZRPrinter   := aZRPrinter;
  WindowState := wsMaximized;
  if ZRPrinter <> nil then Caption := ZRPrinter.Title;
end;

procedure TZRPreviewForm.FormCreate(Sender: TObject);
begin
  Status .Caption:= LoadStr(SzrProcessing);
  Exit   .Caption:= LoadStr(SzrExit);
  Print  .Caption:= LoadStr(SzrPrintReport);
  FirstPage .Hint:= LoadStr(SzrFirstPageHint);
  PrevPage  .Hint:= LoadStr(SzrPrevPageHint);
  NextPage  .Hint:= LoadStr(SzrNextPageHint);
  LastPage  .Hint:= LoadStr(SzrLastPageHint);
  Setup     .Hint:= LoadStr(SzrPrinterSetup);
  Save      .Hint:= LoadStr(SzrSaveReportHint);
  Load      .Hint:= LoadStr(SzrLoadReportHint);
  Print     .Hint:= LoadStr(SzrPrintReportHint);
end;

procedure TZRPreviewForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TZRPreviewForm.UpdateInfo;
begin
  Status.Caption := LoadStr(SzrPage) + ' ' + IntToStr(ZRPreview.PageNumber) + ' ' +
    LoadStr(SzrOf) + ' ' + IntToStr(ZRPreview.ZRPrinter.PageCount);
end;

procedure TZRPreviewForm.FirstPageClick(Sender: TObject);
begin
  ZRPreview.PageNumber := 1;
  UpdateInfo;
end;

procedure TZRPreviewForm.PrevPageClick(Sender: TObject);
begin
  ZRPreview.PageNumber := ZRPreview.PageNumber - 1;
  UpdateInfo;
end;

procedure TZRPreviewForm.NextPageClick(Sender: TObject);
begin
  ZRPreview.PageNumber := ZRPreview.PageNumber + 1;
  UpdateInfo;
end;

procedure TZRPreviewForm.LastPageClick(Sender: TObject);
begin
  ZRPreview.PageNumber := ZRPrinter.PageCount;
  UpdateInfo;
end;

procedure TZRPreviewForm.PrintClick(Sender: TObject);
begin
  with ZRPrinter do if Setup then Print;
end;

procedure TZRPreviewForm.ExitClick(Sender: TObject);
begin
  Close;
end;

procedure TZRPreviewForm.SaveClick(Sender: TObject);
begin
  with TSaveDialog.Create(Application) do
    try
      Filter:= LoadStr(SzrFileFilter);
      if Execute then ZRPrinter.SaveToFile(FileName);
    finally
      Free;
    end;
end;

procedure TZRPreviewForm.SetupClick(Sender: TObject);
begin
  ZRPrinter.Setup;
end;

procedure TZRPreviewForm.LoadClick(Sender: TObject);
begin
{
  with TOpenDialog.Create(Application) do
    try
      if Execute then
        if FileExists(FileName) then
          begin
            ZRPrinter.Load(FileName);
            ZRPreview.PageNumber := 1;
            ZRPreview.PreviewImage.PageNumber := 1;
            UpdateInfo;
          end
        else
          ShowMessage(LoadStr(SzrFileNotExist));
    finally
      free;
    end;
}
end;

procedure TZRPreviewForm.ZRPreviewKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    $50{VK_P}: if Shift=[ssCtrl] then
                 begin
                   Print.Click;
                   Key:= 0;
                 end;
    VK_Next  : if Shift=[] then
                 begin
                   NextPage.Click;
                   Key:= 0;
                 end else
               if Shift=[ssCtrl] then
                 begin
                   LastPage.Click;
                   Key:= 0;
                 end;
    VK_Prior : if Shift=[] then
                 begin
                   PrevPage.Click;
                   Key:= 0;
                 end else
               if Shift=[ssCtrl] then
                 begin
                   FirstPage.Click;
                   Key:= 0;
                 end;
    VK_Up    : with ZRPreview do
               if (PageNumber > 1) and (SelStart = 0) then
                 begin
                   PrevPage.Click;
                   SelStart := GetTextLen;
                   SelLength:= 0;
                   Key:= 0;
                 end;
    VK_Down  : with ZRPreview do
               if (SelStart + SelLength = GetTextLen) then
                 begin
                   NextPage.Click;
                   Key:= 0;
                 end;
    VK_ESCAPE: begin
                 Exit.Click;
                 Key:= 0;
               end;
  end;
end;

procedure TZRPreviewForm.ZRPreviewPageAvailable(Sender: TObject; PageNum: Integer);
begin
  UpdateInfo;
end;

procedure TZRPreviewForm.ZRPreviewProgressUpdate(Sender: TObject; Progress: Integer);
begin
  ProgressBar.Position := Progress;
  ProgressBar.Visible  := (ProgressBar.Position > 0) and (ProgressBar.Position < 100);
end;

function TZRPreviewForm.GetPrinter: TZRPrinter;
begin
  Result:= ZRPreview.ZRPrinter;
end;
procedure TZRPreviewForm.SetPrinter(Value: TZRPrinter);
begin
  ZRPreview.ZRPrinter:= Value;
end;

end.

